@@ -6,11 +6,13 @@ use generated_code::MInst;
 use regalloc::Writable;
 
 // Types that the generated ISLE code uses via `use super::*`.
-use super::{is_mergeable_load, lower_to_amode, Reg};
+use super::{is_int_or_ref_ty, is_mergeable_load, lower_to_amode, Reg};
 use crate::{
     ir::{
-        immediates::*, types::*, Inst, InstructionData, Opcode, TrapCode, Value, ValueLabel,
-        ValueList,
+        condcodes::{FloatCC, IntCC},
+        immediates::*,
+        types::*,
+        Inst, InstructionData, Opcode, TrapCode, Value, ValueLabel, ValueList,
     },
     isa::{
         settings::Flags,
@@ -79,7 +81,7 @@ where
 
         if let Some(c) = inputs.constant {
             if let Some(imm) = to_simm32(c as i64) {
-                return imm;
+                return imm.to_reg_mem_imm();
             }
 
             // Generate constants fresh at each use to minimize long-range
@@ -120,21 +122,23 @@ where
         RegMem::reg(self.put_in_reg(val))
     }
 
-    fn put_masked_in_imm8_reg(&mut self, val: Value, ty: Type) -> Imm8Reg {
+    fn put_masked_in_imm8_gpr(&mut self, val: Value, ty: Type) -> Imm8Gpr {
         let inputs = self.lower_ctx.get_value_as_source_or_const(val);
 
         if let Some(c) = inputs.constant {
             let mask = 1_u64
                 .checked_shl(ty.bits() as u32)
                 .map_or(u64::MAX, |x| x - 1);
-            return Imm8Reg::Imm8 {
+            return Imm8Gpr::new(Imm8Reg::Imm8 {
                 imm: (c & mask) as u8,
-            };
+            })
+            .unwrap();
         }
 
-        Imm8Reg::Reg {
+        Imm8Gpr::new(Imm8Reg::Reg {
             reg: self.put_in_regs(val).regs()[0],
-        }
+        })
+        .unwrap()
     }
 
     #[inline]
@@ -169,6 +173,42 @@ where
         }
     }
 
+    #[inline]
+    fn avx512bitalg_enabled(&mut self, _: Type) -> Option<()> {
+        if self.isa_flags.use_avx512bitalg_simd() {
+            Some(())
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    fn use_lzcnt(&mut self, _: Type) -> Option<()> {
+        if self.isa_flags.use_lzcnt() {
+            Some(())
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    fn use_bmi1(&mut self, _: Type) -> Option<()> {
+        if self.isa_flags.use_bmi1() {
+            Some(())
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    fn use_popcnt(&mut self, _: Type) -> Option<()> {
+        if self.isa_flags.use_popcnt() {
+            Some(())
+        } else {
+            None
+        }
+    }
+
     #[inline]
     fn imm8_from_value(&mut self, val: Value) -> Option<Imm8Reg> {
         let inst = self.lower_ctx.dfg().value_def(val).inst()?;
@@ -178,17 +218,18 @@ where
     }
 
     #[inline]
-    fn const_to_type_masked_imm8(&mut self, c: u64, ty: Type) -> Imm8Reg {
+    fn const_to_type_masked_imm8(&mut self, c: u64, ty: Type) -> Imm8Gpr {
         let mask = 1_u64
             .checked_shl(ty.bits() as u32)
             .map_or(u64::MAX, |x| x - 1);
-        Imm8Reg::Imm8 {
+        Imm8Gpr::new(Imm8Reg::Imm8 {
             imm: (c & mask) as u8,
-        }
+        })
+        .unwrap()
     }
 
     #[inline]
-    fn simm32_from_value(&mut self, val: Value) -> Option<RegMemImm> {
+    fn simm32_from_value(&mut self, val: Value) -> Option<GprMemImm> {
         let inst = self.lower_ctx.dfg().value_def(val).inst()?;
         let constant: u64 = self.lower_ctx.get_constant(inst)?;
         let constant = constant as i64;
@@ -196,7 +237,7 @@ where
     }
 
     #[inline]
-    fn simm32_from_imm64(&mut self, imm: Imm64) -> Option<RegMemImm> {
+    fn simm32_from_imm64(&mut self, imm: Imm64) -> Option<GprMemImm> {
         to_simm32(imm.bits())
     }
 
@@ -323,6 +364,16 @@ where
         SyntheticAmode::ConstantOffset(mask_table)
     }
 
+    fn popcount_4bit_table(&mut self) -> VCodeConstant {
+        self.lower_ctx
+            .use_constant(VCodeConstantData::WellKnown(&POPCOUNT_4BIT_TABLE))
+    }
+
+    fn popcount_low_mask(&mut self) -> VCodeConstant {
+        self.lower_ctx
+            .use_constant(VCodeConstantData::WellKnown(&POPCOUNT_LOW_MASK))
+    }
+
     #[inline]
     fn writable_reg_to_xmm(&mut self, r: WritableReg) -> WritableXmm {
         Writable::from_reg(Xmm::new(r.to_reg()).unwrap())
@@ -412,6 +463,62 @@ where
     fn reg_to_gpr_mem(&mut self, r: Reg) -> GprMem {
         GprMem::new(RegMem::reg(r)).unwrap()
     }
+
+    #[inline]
+    fn imm8_reg_to_imm8_gpr(&mut self, ir: &Imm8Reg) -> Imm8Gpr {
+        Imm8Gpr::new(ir.clone()).unwrap()
+    }
+
+    #[inline]
+    fn gpr_to_gpr_mem(&mut self, gpr: Gpr) -> GprMem {
+        GprMem::from(gpr)
+    }
+
+    #[inline]
+    fn gpr_to_gpr_mem_imm(&mut self, gpr: Gpr) -> GprMemImm {
+        GprMemImm::from(gpr)
+    }
+
+    #[inline]
+    fn gpr_to_imm8_gpr(&mut self, gpr: Gpr) -> Imm8Gpr {
+        Imm8Gpr::from(gpr)
+    }
+
+    #[inline]
+    fn imm8_to_imm8_gpr(&mut self, imm: u8) -> Imm8Gpr {
+        Imm8Gpr::new(Imm8Reg::Imm8 { imm }).unwrap()
+    }
+
+    fn is_gpr_type(&mut self, ty: Type) -> Option<Type> {
+        if is_int_or_ref_ty(ty) || ty == I128 || ty == B128 {
+            Some(ty)
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    fn is_xmm_type(&mut self, ty: Type) -> Option<Type> {
+        if ty == F32 || ty == F64 || (ty.is_vector() && ty.bits() == 128) {
+            Some(ty)
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    fn is_single_register_type(&mut self, ty: Type) -> Option<Type> {
+        if ty != I128 {
+            Some(ty)
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    fn intcc_to_cc(&mut self, intcc: &IntCC) -> CC {
+        CC::from_intcc(*intcc)
+    }
 }
 
 // Since x64 doesn't have 8x16 shifts and we must use a 16x8 shift instead, we
@@ -445,12 +552,27 @@ const I8X16_USHR_MASKS: [u8; 128] = [
     0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
 ];
 
+/// Number of bits set in a given nibble (4-bit value). Used in the
+/// vector implementation of popcount.
+#[rustfmt::skip] // Preserve 4x4 layout.
+const POPCOUNT_4BIT_TABLE: [u8; 16] = [
+    0x00, 0x01, 0x01, 0x02,
+    0x01, 0x02, 0x02, 0x03,
+    0x01, 0x02, 0x02, 0x03,
+    0x02, 0x03, 0x03, 0x04,
+];
+
+const POPCOUNT_LOW_MASK: [u8; 16] = [0x0f; 16];
+
 #[inline]
-fn to_simm32(constant: i64) -> Option<RegMemImm> {
+fn to_simm32(constant: i64) -> Option<GprMemImm> {
     if constant == ((constant << 32) >> 32) {
-        Some(RegMemImm::Imm {
-            simm32: constant as u32,
-        })
+        Some(
+            GprMemImm::new(RegMemImm::Imm {
+                simm32: constant as u32,
+            })
+            .unwrap(),
+        )
     } else {
         None
     }
