@@ -13,8 +13,8 @@ use crate::ir::instructions::BranchInfo;
 use crate::ir::{
     types::{FFLAGS, IFLAGS},
     ArgumentPurpose, Block, Constant, ConstantData, DataFlowGraph, ExternalName, Function,
-    GlobalValueData, Inst, InstructionData, MemFlags, Opcode, Signature, SourceLoc, Type, Value,
-    ValueDef, ValueLabelAssignments, ValueLabelStart,
+    GlobalValue, GlobalValueData, Inst, InstructionData, MemFlags, Opcode, Signature, SourceLoc,
+    Type, Value, ValueDef, ValueLabelAssignments, ValueLabelStart,
 };
 use crate::machinst::{
     non_writable_value_regs, writable_value_regs, ABICallee, BlockIndex, BlockLoweringOrder,
@@ -93,6 +93,11 @@ pub trait LowerCtx {
     /// Get the symbol name, relocation distance estimate, and offset for a
     /// symbol_value instruction.
     fn symbol_value<'b>(&'b self, ir_inst: Inst) -> Option<(&'b ExternalName, RelocDistance, i64)>;
+    /// Likewise, but starting with a GlobalValue identifier.
+    fn symbol_value_data<'b>(
+        &'b self,
+        global_value: GlobalValue,
+    ) -> Option<(&'b ExternalName, RelocDistance, i64)>;
     /// Returns the memory flags of a given memory access.
     fn memflags(&self, ir_inst: Inst) -> Option<MemFlags>;
     /// Get the source location for a given instruction.
@@ -328,14 +333,10 @@ fn alloc_vregs<I: VCodeInst>(
     let regs = match regclasses {
         &[rc0] => ValueRegs::one(Reg::new_virtual(rc0, v)),
         &[rc0, rc1] => ValueRegs::two(Reg::new_virtual(rc0, v), Reg::new_virtual(rc1, v + 1)),
-        #[cfg(feature = "arm32")]
-        &[rc0, rc1, rc2, rc3] => ValueRegs::four(
-            Reg::new_virtual(rc0, v),
-            Reg::new_virtual(rc1, v + 1),
-            Reg::new_virtual(rc2, v + 2),
-            Reg::new_virtual(rc3, v + 3),
-        ),
-        _ => panic!("Value must reside in 1, 2 or 4 registers"),
+        // We can extend this if/when we support 32-bit targets; e.g.,
+        // an i128 on a 32-bit machine will need up to four machine regs
+        // for a `Value`.
+        _ => panic!("Value must reside in 1 or 2 registers"),
     };
     for (&reg_ty, &reg) in tys.iter().zip(regs.regs().iter()) {
         vcode.set_vreg_type(reg.to_virtual_reg(), reg_ty);
@@ -1070,19 +1071,26 @@ impl<'func, I: VCodeInst> LowerCtx for Lower<'func, I> {
     fn symbol_value<'b>(&'b self, ir_inst: Inst) -> Option<(&'b ExternalName, RelocDistance, i64)> {
         match &self.f.dfg[ir_inst] {
             &InstructionData::UnaryGlobalValue { global_value, .. } => {
-                let gvdata = &self.f.global_values[global_value];
-                match gvdata {
-                    &GlobalValueData::Symbol {
-                        ref name,
-                        ref offset,
-                        ..
-                    } => {
-                        let offset = offset.bits();
-                        let dist = gvdata.maybe_reloc_distance().unwrap();
-                        Some((name, dist, offset))
-                    }
-                    _ => None,
-                }
+                self.symbol_value_data(global_value)
+            }
+            _ => None,
+        }
+    }
+
+    fn symbol_value_data<'b>(
+        &'b self,
+        global_value: GlobalValue,
+    ) -> Option<(&'b ExternalName, RelocDistance, i64)> {
+        let gvdata = &self.f.global_values[global_value];
+        match gvdata {
+            &GlobalValueData::Symbol {
+                ref name,
+                ref offset,
+                ..
+            } => {
+                let offset = offset.bits();
+                let dist = gvdata.maybe_reloc_distance().unwrap();
+                Some((name, dist, offset))
             }
             _ => None,
         }
@@ -1093,10 +1101,8 @@ impl<'func, I: VCodeInst> LowerCtx for Lower<'func, I> {
             &InstructionData::AtomicCas { flags, .. } => Some(flags),
             &InstructionData::AtomicRmw { flags, .. } => Some(flags),
             &InstructionData::Load { flags, .. }
-            | &InstructionData::LoadComplex { flags, .. }
             | &InstructionData::LoadNoOffset { flags, .. }
-            | &InstructionData::Store { flags, .. }
-            | &InstructionData::StoreComplex { flags, .. } => Some(flags),
+            | &InstructionData::Store { flags, .. } => Some(flags),
             &InstructionData::StoreNoOffset { flags, .. } => Some(flags),
             _ => None,
         }
